/* This file is part of the implementation of the Dynamic Neuaral Network
   for speech recognition.

   Copyright (C) 1998 Lin Zhong<lzhong@princeton.edu>
                      Runsheng Liu <lrs-dee@tsinghua.edu.cn>
                      Circuits & Systems Group
                      Dept. of Electronic Engineering
                      Tsinghua University, Beijing
   Copyright (C) 2004 Lin Zhong <lzhong@princeton.edu>
                      Computer Engineering Group
                      Dept. of Electrical Engineering
                      Princeton University

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software. In case the
software is used in a publication, please cite
Lin Zhong, Yuanyuan Shi and Runsheng Liu,
"A Dynamic Neural Network for Syllable Recognition",
in Proc. Intl. Joint Conf. Neural Networks, Washington D.C., 1999.


THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <math.h>
#include <assert.h>
#include "NN.h"
nnStructParam::nnStructParam(void) {
        inputTimeWindow = 5;
        hiddenLayerTimeWindow = 5;
        inputFeatureWindow = 8;
        type = DNN;
        numClasses = 0;
        numFeatures = 12;
        numHiddenNeurons = 12;
	return;
}
void nnStructParam::Dump(FILE *fp) {
	fprintf(fp, "%u %u %u %u ",type, inputTimeWindow,hiddenLayerTimeWindow,numClasses);
	fprintf(fp, "%u %u %u \n", numClasses, numFeatures, numHiddenNeurons);
	return;
}
void nnStructParam::Read(FILE *fp) {
	unsigned int tmpType;
	fscanf(fp, "%u %u %u %u ",&tmpType, &inputTimeWindow,&hiddenLayerTimeWindow,&numClasses);
	fscanf(fp, "%u %u %u\n", &numClasses, &numFeatures, &numHiddenNeurons);
	type = (NN_Type)tmpType;
	return;
}
	
nnTrainParam::nnTrainParam(void){
        dLow = 0.2;
        dHigh = 0.8;
        error = 0.1;
        alpha = 0.05;
        ita = 0;
        maxEpoch = 200;
        maxLoop = 20;
	scale = 1.0;
}
nnTrainParam::~nnTrainParam() {
/*
	unsigned int i, j ;
	for(i = 0; i<MAX_CLASSES;i++) {
		if(trainData[i]) {
			for(j = 0;j<numObservations[i];j++){
				if(trainData[i][j]){
					delete[] trainData[i][j];
				}
			}			
			delete[] trainData[i];
		}
	}
*/
	return;
}	
void nnTrainParam::ShowParameters(void){
	printf("dLow=%.2f dHigh=%.2f error=%.2f\n",dLow,dHigh, error);
	printf("Alpha=%.2f Ita=%.2f EpochNum=%d LoopNum=%d\n", alpha, ita, maxEpoch, maxLoop);
	return;
}
NN::NN(nnStructParam &StructParam){
	nnStructure = &StructParam;

	return;
}
NN::NN(nnStructParam &StructParam, nnTrainParam &TrainParam ) {
	nnStructure = &StructParam;
	nnTraining = &TrainParam;

	SetMethods(nnStructure->type);
	return;
}
NN::~NN()
{
//	(*this.*memfree)();
}
void NN::SetMethods(NN_Type type){
        switch(type) {
          case DNN:
                memfree = &NN::DNN_memfree;
                Train = &NN::DNN_Train;
                Save = &NN::DNN_Save;
		Load = &NN::DNN_Load;
		Recognize = &NN::DNN_Recognize;
                break;
          case MLP:
                memfree = &NN::MLP_memfree;
                Train = &NN::MLP_Train;
                Save = &NN::MLP_Save;
		Load = &NN::MLP_Load;
		Recognize = &NN::MLP_Recognize;
          case PER:
                memfree = &NN::PER_memfree;
                Train = &NN::PER_Train;
                Save = &NN::PER_Save;
		Load = &NN::PER_Load;
		Recognize = &NN::PER_Recognize;
                break;
          default:
                memfree = &NN::DNN_memfree;
                Train = &NN::DNN_Train;
                Save = &NN::DNN_Save;
		Load = &NN::DNN_Load;
		Recognize = &NN::DNN_Recognize;
                break;
        }
        return;
}
void NN::NNLoad(char *fileName){
	FILE *fp;
	if((fp=fopen(fileName, "r"))==NULL) {
		fprintf(stderr,"%s: ", fileName);
		ErrorMessage("Cannot open NN data file\n");
	}
	nnStructure->Read(fp);
	SetMethods(nnStructure->type);
	(*this.*Load)(fp);
	fclose(fp);
	return;
}

unsigned int NN::NNRecognize(float **input, unsigned int inputLength)
{
	return (*this.*Recognize)(input, inputLength);
}	
unsigned int NN::NNTrain(void) {
	
	return (*this.*Train)();
}
unsigned int NN::NNSave(char *fileName) {
	FILE *fp;
	unsigned int res;
	if((fp=fopen(fileName, "w"))==NULL){
		fprintf(stderr,"%s: ", fileName);
		ErrorMessage("Cannot create NN data file");
	}
	res = (*this.*Save)(fp);
	fclose(fp);
	return res;
}
unsigned int NN::DNN_Init(void) {

  unsigned int   k,i,j;

  w1   =  new float **[nnStructure->inputTimeWindow];
  assert(w1);
  w2  =  new float ***[nnStructure->hiddenLayerTimeWindow];
  assert(w2);
  b1    =  new float[nnStructure->numHiddenNeurons];
  assert(b1);
  b2    =  new float[nnStructure->numClasses];
  assert(b2);
  for(i=0;i<nnStructure->inputTimeWindow;i++)
  {
	  w1[i]     =  new float *[nnStructure->numHiddenNeurons];
	  assert(w1[i]);
	  for(j=0;j<nnStructure->numHiddenNeurons;j++){
	    w1[i][j]=  new float[nnStructure->inputFeatureWindow];
	    assert(w1[i][j]);
	  }
  }
  for(i=0;i<nnStructure->hiddenLayerTimeWindow;i++)
  {
	  w2[i]     =  new float **[nnStructure->numClasses];
	  assert(w2[i]);
	  for(j=0;j<nnStructure->numClasses;j++)
	  {
		  w2[i][j]=  new float *[nnStructure->numHiddenNeurons];
		  assert(w2[i][j]);
		  for(k=0;k<nnStructure->numHiddenNeurons;k++) {
               		w2[i][j][k]=  new float[(nnStructure->numFeatures-nnStructure->inputFeatureWindow+1)];
			assert(w2[i][j][k]);
		  }
	  }
  
  }
  return 1;
}
unsigned int NN::DNN_InitTrain(void) {
   unsigned int i, j, k, t;
   srand( (unsigned)time( NULL ) );
   for(i=0;i<nnStructure->numHiddenNeurons;i++)
   {
        for(j=0;j<nnStructure->inputTimeWindow;j++)
	       for(k=0;k<nnStructure->inputFeatureWindow;k++)
			  w1[j][i][k]=(float)(rand()-RAND_MAX*0.5)/(nnTraining->scale*RAND_MAX);
	    b1[i]=(float)(rand()-RAND_MAX*0.5)/(nnTraining->scale*RAND_MAX);
   }
   for(i=0;i<nnStructure->numClasses;i++)
   {
	    for(j=0;j<nnStructure->hiddenLayerTimeWindow;j++)
	       for(k=0;k<nnStructure->numHiddenNeurons;k++)
			for(t=0;t<(nnStructure->numFeatures-nnStructure->inputFeatureWindow+1);t++)
			   w2[j][i][k][t]=(float)(rand()-RAND_MAX*0.5)/(nnTraining->scale*RAND_MAX);
	    b2[i]=(float)(rand()-RAND_MAX*0.5)/(nnTraining->scale*RAND_MAX);
   }
   
 return 1;
}

 unsigned int NN::DNN_Train(void)
{
  
  unsigned int i,j,k,l,m,n,t;
  unsigned int classNum,epoch,itr;
  float temp;
  float alpha,ita;
   DNN_Init();
   DNN_InitTrain();
  m=0;
  for(i=0;i<nnStructure->numClasses;i++){
     if(m<nnTraining->numObservations[i])
		 m=nnTraining->numObservations[i];
  }
  float **target    =    new float *[nnStructure->numClasses];
  assert(target);
  for(i=0;i<nnStructure->numClasses;i++) {   
	  target[i]= new float[nnStructure->numClasses];
	  assert(target[i]);
	  for(j=0;j<nnStructure->numClasses;j++){
	  	  target[i][j]=nnTraining->dLow;
	  }
      	  target[i][i]=nnTraining->dHigh;
  }
  float *o          =    new float[nnStructure->numClasses];
  assert(o);
  float ***I1       =    new float **[m];
  assert(I1);
  float ***delta1   =    new float **[m];
  assert(delta1);
  for(i=0;i<m;i++) {
	  I1[i]     =  new float *[nnStructure->numHiddenNeurons];
	  assert(I1[i]);
	  delta1[i] =  new float *[nnStructure->numHiddenNeurons];
	  assert(delta1[i]);
          for(j=0;j<nnStructure->numHiddenNeurons;j++)
	  {
		  I1[i][j]     =  new float[(nnStructure->numFeatures-nnStructure->inputFeatureWindow+1)];
		  assert(I1[i][j]);
	          delta1[i][j] =  new float[(nnStructure->numFeatures-nnStructure->inputFeatureWindow+1)];
		  assert(delta1[i][j]);
      	  }
  }
  
  float **I2       =    new float *[m];
  assert(I2);
  float **delta2   =    new float *[m];
  assert(delta2);
  for(i=0;i<m;i++) {
	  I2[i]     =  new float[nnStructure->numClasses];
	  assert(I2[i]);
	  delta2[i] =  new float[nnStructure->numClasses];
	  assert(delta2[i]);
  }
  float ***delw1   =  new float **[nnStructure->inputTimeWindow];
  assert(delw1); 
  float ****delw2  =  new float ***[nnStructure->hiddenLayerTimeWindow];
  assert(delw2);
  float  *delb1    =  new float[nnStructure->numHiddenNeurons];
  assert(delb1);
  float  *delb2    =  new float[nnStructure->numClasses];
  assert(delb2);
  for(i=0;i<nnStructure->inputTimeWindow;i++) {
	  delw1[i]     =  new float *[nnStructure->numHiddenNeurons];
	  assert(delw1[i]);
	  for(j=0;j<nnStructure->numHiddenNeurons;j++) {
	    delw1[i][j]=  new float[nnStructure->inputFeatureWindow];
	    assert(delw1[i][j]);
	  }
  }
  for(i=0;i<nnStructure->hiddenLayerTimeWindow;i++) {
	  delw2[i]     =  new float **[nnStructure->numClasses];
	  assert(delw2[i]);
	  for(j=0;j<nnStructure->numClasses;j++)
	  {
		  delw2[i][j]=  new float *[nnStructure->numHiddenNeurons];
		  assert(delw2[i][j]);
		  for(k=0;k<nnStructure->numHiddenNeurons;k++) {
                  	delw2[i][j][k]=  new float[(nnStructure->numFeatures-nnStructure->inputFeatureWindow+1)];
			assert(delw2[i][j][k]);
		  }
	  }
  
  }

  unsigned int max_itr=100;


  for(epoch=0;epoch<nnTraining->maxEpoch;epoch++)
  {
  
    if(max_itr==0)
	  break;
    max_itr=0;
  
    for(classNum=0;classNum<nnStructure->numClasses;classNum++)
	{
	  for(k=0;k<nnStructure->inputTimeWindow;k++)
        for(j=0;j<nnStructure->numHiddenNeurons;j++)
          for(l=0;l<nnStructure->inputFeatureWindow;l++)
	           delw1[k][j][l]=0;
      for(k=0;k<nnStructure->hiddenLayerTimeWindow;k++)
        for(j=0;j<nnStructure->numClasses;j++)
          for(l=0;l<nnStructure->numHiddenNeurons;l++)
           for(t=0;t<(nnStructure->numFeatures-nnStructure->inputFeatureWindow+1);t++)
		       delw2[k][j][l][t]=0;
      for(i=0;i<nnStructure->numHiddenNeurons;i++)
	       delb1[i]=0;
      for(i=0;i<nnStructure->numClasses;i++)
	       delb2[i]=0;
      for(itr=0;itr<nnTraining->maxLoop;itr++) 
	  {    
           
         alpha=nnTraining->alpha*(1+0.5/(itr+1+log(epoch)));
         ita  =nnTraining->ita*(1+0.2/(itr+1+log(epoch)));
         for(i=0;i<nnTraining->numObservations[classNum];i++)
	     for(j=0;j<nnStructure->numHiddenNeurons;j++)
	       for(k=0;k<(nnStructure->numFeatures-nnStructure->inputFeatureWindow+1);k++)
		   {
		       I1[i][j][k]     =   b1[j];
              	       delta1[i][j][k] =   0;
		   }
        for(i=0;i<nnTraining->numObservations[classNum];i++)
	       for(j=0;j<nnStructure->numClasses;j++)
		   {
		       I2[i][j] = b2[j];
		       delta2[i][j]=0;
		   } 
   
   //I1=w1*input;
	    for(i=0;i<(nnTraining->numObservations[classNum]-nnStructure->inputTimeWindow+1);i++)
	       for(j=0;j<nnStructure->numHiddenNeurons;j++)
	         for(t=0;t<(nnStructure->numFeatures-nnStructure->inputFeatureWindow+1);t++) {
		        for(k=0;k<nnStructure->inputTimeWindow;k++)
		          for(l=0;l<nnStructure->inputFeatureWindow;l++)
			         I1[i][j][t]+=w1[k][j][l]*nnTraining->trainData[classNum][i+k][t+l];
                	I1[i][j][t] = logsig(I1[i][j][t]);
			 }
// I2=w2*I1;
        for(i=0;i<(nnTraining->numObservations[classNum]-nnStructure->inputTimeWindow-nnStructure->hiddenLayerTimeWindow+2);i++)
	       for(j=0;j<nnStructure->numClasses;j++)
		   {
		       for(k=0;k<nnStructure->hiddenLayerTimeWindow;k++)
		           for(l=0;l<nnStructure->numHiddenNeurons;l++)
			         for(t=0;t<(nnStructure->numFeatures-nnStructure->inputFeatureWindow+1);t++)
			          I2[i][j]+=w2[k][j][l][t]*I1[i+k][l][t];
               I2[i][j] = logsig(I2[i][j]);
		   }
        float   E = 0.0;
        for(i=0;i<nnStructure->numClasses;i++)
		{
	      o[i] = 0.0;
	      for(j=0;j<(nnTraining->numObservations[classNum]-nnStructure->hiddenLayerTimeWindow-nnStructure->inputTimeWindow+2);j++)
		   o[i]+=  log(I2[j][i]);
          o[i]/=(nnTraining->numObservations[classNum]-nnStructure->hiddenLayerTimeWindow-nnStructure->inputTimeWindow+2);   
	      o[i]=exp(o[i]);
	      E+=(o[i]-target[i][classNum])*(o[i]-target[i][classNum]);
		}
        E/=1.0;
        E=sqrt(E/nnStructure->numClasses);
        if(E<=nnTraining->error)
    	 	 break;

   
   //the second layer;
        for(i=0;i<(nnTraining->numObservations[classNum]-nnStructure->inputTimeWindow-nnStructure->hiddenLayerTimeWindow+2);i++)
        	for(j=0;j<nnStructure->numClasses;j++)
			{
		       delta2[i][j]=-(o[j]-target[j][classNum])*(1-I2[i][j])/(nnTraining->numObservations[classNum]-nnStructure->inputTimeWindow-nnStructure->hiddenLayerTimeWindow+2);
		       delta2[i][j]/=(1-o[j]);
			} 

	
        for(j=0;j<nnStructure->numClasses;j++)
		{
	       temp=0;
	       for(i=0;i<(nnTraining->numObservations[classNum]-nnStructure->inputTimeWindow-nnStructure->hiddenLayerTimeWindow+2);i++)
	 	      temp+= delta2[i][j];
	       delb2[j]    =  alpha*temp+ita*delb2[j];
	       for(k=0;k<nnStructure->hiddenLayerTimeWindow;k++)
              for(l=0;l<nnStructure->numHiddenNeurons;l++)
	             for(t=0;t<(nnStructure->numFeatures-nnStructure->inputFeatureWindow+1);t++)
				 {
		            temp=0;
		            for(i=0;i<(nnTraining->numObservations[classNum]-k-nnStructure->inputTimeWindow-nnStructure->hiddenLayerTimeWindow+2);i++)
			            temp+=delta2[i][j]*I1[i+k][l][t];
		            delw2[k][j][l][t]=alpha*temp+ita*delw2[k][j][l][t];
				 }   
		}// end of j

   // the first layer;
        for(i=0;i<(nnTraining->numObservations[classNum]-nnStructure->inputTimeWindow+1);i++)
	        for(j=0;j<nnStructure->numHiddenNeurons;j++)
	         for(t=0;t<(nnStructure->numFeatures-nnStructure->inputFeatureWindow+1);t++)
			 {  
		         if((i-nnStructure->hiddenLayerTimeWindow+1)<0)
					 m=0;
				 else
					 m=i-nnStructure->hiddenLayerTimeWindow+1;
				 temp=0;
		         for(;m<=i;m++)
		            for(n=0;n<nnStructure->numClasses;n++)
		                 temp+=delta2[m][n]*w2[i-m][n][j][t];
		         delta1[i][j][t]=(1-I1[i][j][t])*I1[i][j][t]*temp;
	    
		   
			 }  
   
        for(j=0;j<nnStructure->numHiddenNeurons;j++)
	{
 	        temp=0;
	        for(i=0;i<(nnTraining->numObservations[classNum]-nnStructure->inputTimeWindow+1);i++)
               		for(t=0;t<(nnStructure->numFeatures-nnStructure->inputFeatureWindow+1);t++)
		           temp+= delta1[i][j][t];
	         delb1[j]    =  alpha*temp+ita*delb1[j];
	
	        for(k=0;k<nnStructure->inputTimeWindow;k++)
               		for(l=0;l<nnStructure->inputFeatureWindow;l++)
	 		{
		         	temp=0;
		 		 
                 		for(t=0;t<(nnStructure->numFeatures-nnStructure->inputFeatureWindow+1);t++)
		            	for(i=0;i<(nnTraining->numObservations[classNum]-nnStructure->inputTimeWindow+1);i++)//??????????
			           temp+=delta1[i][j][t]*nnTraining->trainData[classNum][i+k][t+l];
		         delw1[k][j][l]=alpha*temp+ita*delw1[k][j][l];
		 
		 
			   } 
		}// end of j 
   
   //update w1,w2,b1,b2;
        for(k=0;k<nnStructure->inputTimeWindow;k++)
   	      for(j=0;j<nnStructure->numHiddenNeurons;j++)
	        for(l=0;l<nnStructure->inputFeatureWindow;l++)
    	       w1[k][j][l]+=delw1[k][j][l];
	
        for(k=0;k<nnStructure->hiddenLayerTimeWindow;k++)
  	      for(j=0;j<nnStructure->numClasses;j++)
	        for(l=0;l<nnStructure->numHiddenNeurons;l++)
	           for(t=0;t<(nnStructure->numFeatures-nnStructure->inputFeatureWindow+1);t++) 
	               w2[k][j][l][t]+=delw2[k][j][l][t];
       
        for(i=0;i<nnStructure->numHiddenNeurons;i++)
	          b1[i]+=delb1[i];
        for(i=0;i<nnStructure->numClasses;i++)
	          b2[i]+=delb2[i];
 
    }//end iteration;
    
  if(itr>max_itr)
     max_itr=itr;
 
   }//loop classNum end 

 }  //loop epoch end  
  for(i=0;i<nnStructure->numClasses;i++){
     if(m<nnTraining->numObservations[i])
		 m=nnTraining->numObservations[i];
  }
  for(i=0;i<m;i++)
  {
	  for(j=0;j<nnStructure->numHiddenNeurons;j++)
	  {
		  delete[] I1[i][j];
		  delete[] delta1[i][j];
	  }
	  delete[] I1[i]  ;
	  delete[] delta1[i];
  }
  delete[] I1;
  delete[] delta1;
  for(i=0;i<m;i++)
  {
	  delete[] I2[i] ;
	  delete[] delta2[i] ;
  }
  delete[] I2;
  delete[] delta2;
  for(i=0;i<nnStructure->inputTimeWindow;i++)
  {
	 for(j=0;j<nnStructure->numHiddenNeurons;j++)
	   delete[] delw1[i][j];
	 delete[] delw1[i];
  }
 delete[] delw1;
 delete[] delb1;
  for(i=0;i<nnStructure->hiddenLayerTimeWindow;i++)
  {
	  for(j=0;j<nnStructure->numClasses;j++)
	  {
		  for(k=0;k<nnStructure->numHiddenNeurons;k++)
	    	  delete[] delw2[i][j][k];
	      delete[] delw2[i][j];
	  }
	  delete[] delw2[i];
  }
 delete[] delw2; 
 delete[] delb2;
 delete[] o;
 for(i=0;i<nnStructure->numClasses;i++)
	 delete[] target[i];
 delete[] target;
  if(epoch==nnTraining->maxEpoch)
  {
      DNN_memfree();
      ErrorMessage("Training Failed!");
  }  
  
 return 1;
 }

 float NN::logsig(float x)
{
 if(x<-10000)
    {
	  ErrorMessage(" Training Overfloat!");
	 exit(1);
    }
 else
 return 1/(1+exp(-x));
}

 unsigned int NN::DNN_Recognize(float **input, unsigned int inputLength)
{
    unsigned int i,j,k,l,t;
    unsigned int label;
    float temp;
    float *o   =  new float[nnStructure->numClasses];
    float ***I1       =    new float **[inputLength-nnStructure->inputTimeWindow+1];
    for(i=0;i<(inputLength-nnStructure->inputTimeWindow+1);i++)
    {
 	  I1[i]     =  new float *[nnStructure->numHiddenNeurons];
          for(j=0;j<nnStructure->numHiddenNeurons;j++)
		  I1[i][j]     =  new float[(nnStructure->numFeatures-nnStructure->inputFeatureWindow+1)];
	}
  
    float **I2       =    new float *[inputLength-nnStructure->inputTimeWindow-nnStructure->hiddenLayerTimeWindow+2];
    for(i=0;i<(inputLength-nnStructure->inputTimeWindow-nnStructure->hiddenLayerTimeWindow+2);i++)
	  I2[i]     =  new float[nnStructure->numClasses];
	
  
  for(i=0;i<(inputLength-nnStructure->inputTimeWindow+1);i++)
    for(j=0;j<nnStructure->numHiddenNeurons;j++)
	   for(k=0;k<(nnStructure->numFeatures-nnStructure->inputFeatureWindow+1);k++)
		   I1[i][j][k]     =   b1[j];
  for(i=0;i<(inputLength-nnStructure->inputTimeWindow-nnStructure->hiddenLayerTimeWindow+2);i++)
    for(j=0;j<nnStructure->numClasses;j++)
	   I2[i][j] = b2[j];
 
   //I1=w1*input;
   for(i=0;i<(inputLength-nnStructure->inputTimeWindow+1);i++)
    for(j=0;j<nnStructure->numHiddenNeurons;j++)
	  for(t=0;t<(nnStructure->numFeatures-nnStructure->inputFeatureWindow+1);t++)
	  {
		 for(k=0;k<nnStructure->inputTimeWindow;k++)
		   for(l=0;l<nnStructure->inputFeatureWindow;l++)
			   I1[i][j][t]+=w1[k][j][l]*input[i+k][t+l];
         	 I1[i][j][t] = logsig(I1[i][j][t]);
	  }
// I2=w2*I1;
   for(i=0;i<(inputLength-nnStructure->inputTimeWindow-nnStructure->hiddenLayerTimeWindow+2);i++)
	 for(j=0;j<nnStructure->numClasses;j++)
	 {
		 for(k=0;k<nnStructure->hiddenLayerTimeWindow;k++)
		   for(l=0;l<nnStructure->numHiddenNeurons;l++)
			 for(t=0;t<(nnStructure->numFeatures-nnStructure->inputFeatureWindow+1);t++)
			   I2[i][j]+=w2[k][j][l][t]*I1[i+k][l][t];
         	 I2[i][j] = logsig(I2[i][j]);
	 }
// winner_take_all
    temp=0; 
	for(i=0;i<nnStructure->numClasses;i++)
	{
	   o[i] = 0;
	   for(j=0;j<(inputLength-nnStructure->inputTimeWindow-nnStructure->hiddenLayerTimeWindow+2);j++)
		 o[i]+=  log(I2[j][i]);
       o[i]/=(inputLength-nnStructure->inputTimeWindow-nnStructure->hiddenLayerTimeWindow+2);   
	   o[i]=exp(o[i]);
	   if(o[i]>temp)
	   {
		   temp=o[i];
		   label=i;
	   }

	}
    delete[] o;
	for(i=0;i<(inputLength-nnStructure->inputTimeWindow-nnStructure->hiddenLayerTimeWindow+2);i++)
		delete[] I2[i];
	delete[] I2;
    for(i=0;i<(inputLength-nnStructure->inputTimeWindow+1);i++)
	{
		for(j=0;j<nnStructure->numHiddenNeurons;j++)
			delete[] I1[i][j];
		delete[] I1[i];
	}
	delete[] I1;
 	return label;
}
unsigned int NN::DNN_Save(FILE *fp){
	unsigned int i, j, k,l;	
	nnStructure->Dump(fp);
	fprintf(fp, "\n");
	assert(w1);
	for(i=0;i<nnStructure->inputTimeWindow;i++){
	 assert(w1[i]);
	 for(j=0;j<nnStructure->numHiddenNeurons;j++){
	  assert(w1[i][j]);
	  for(k=0;k<nnStructure->inputFeatureWindow;k++){
		fprintf(fp, "%f\t",w1[i][j][k]);
	  }
	 }
	}
        assert(w2);
        for(i=0;i<nnStructure->hiddenLayerTimeWindow;i++){
         assert(w2[i]);
         for(j=0;j<nnStructure->numClasses;j++){
          assert(w2[i][j]);
          for(k=0;k<nnStructure->numHiddenNeurons;k++){
	   assert(w2[i][j][k]);
	   for(l=0;l<(nnStructure->numFeatures-nnStructure->inputFeatureWindow+1);l++){
                fprintf(fp, "%f\t",w2[i][j][k][l]);
	   }
          }
         }
        }
	assert(b1);
	for(i=0;i<nnStructure->numHiddenNeurons;i++) {
		fprintf(fp, "%f\t",b1[i]);
	}
        assert(b2);
        for(i=0;i<nnStructure->numClasses;i++) {
                fprintf(fp, "%f\t",b2[i]);
        }

	return 1;
}
unsigned int NN::DNN_Load(FILE *fp) {
        unsigned int i, j, k,l;
	DNN_Init();
	nnStructure->Read(fp);
        assert(w1);
        for(i=0;i<nnStructure->inputTimeWindow;i++){
         assert(w1[i]);
         for(j=0;j<nnStructure->numHiddenNeurons;j++){
          assert(w1[i][j]);
          for(k=0;k<nnStructure->inputFeatureWindow;k++){
                fscanf(fp, "%f\t",&w1[i][j][k]);
          }
         }
        }
        assert(w2);
        for(i=0;i<nnStructure->hiddenLayerTimeWindow;i++){
         assert(w2[i]);
         for(j=0;j<nnStructure->numClasses;j++){
          assert(w2[i][j]);
          for(k=0;k<nnStructure->numHiddenNeurons;k++){
           assert(w2[i][j][k]);
           for(l=0;l<(nnStructure->numFeatures-nnStructure->inputFeatureWindow+1);l++){
                fscanf(fp, "%f\t",&w2[i][j][k][l]);
           }
          }
         }
        }
        assert(b1);
        for(i=0;i<nnStructure->numHiddenNeurons;i++) {
                fscanf(fp, "%f\t",&b1[i]);
        }
        assert(b2);
        for(i=0;i<nnStructure->numClasses;i++) {
                fscanf(fp, "%f\t",&b2[i]);
        }

	return 1;
}
 void NN::DNN_memfree(void)
{
	unsigned int i,j,k;   
	for(i=0;i<nnStructure->inputTimeWindow;i++)
	{
	   for(j=0;j<nnStructure->numHiddenNeurons;j++)
	      delete[] w1[i][j];
	   delete[] w1[i];
	} 
    	delete[] w1;
    	delete[] b1;
	for(i=0;i<nnStructure->hiddenLayerTimeWindow;i++)
	{ 
	  for(j=0;j<nnStructure->numClasses;j++)
	  {
		  for(k=0;k<nnStructure->numHiddenNeurons;k++)
		    delete[] w2[i][j][k];
	      delete[] w2[i][j];
	  }
	  delete[] w2[i];
	} 
    	delete[] w2; 
    	delete[] b2;
return;
}

unsigned int NN::MLP_Init()
{
 
  unsigned int i;
 
  mlp_w1  =  new float *[nnStructure->numHiddenNeurons];
  for(i=0;i<nnStructure->numHiddenNeurons;i++)
      mlp_w1[i] =  new float[nnStructure->numFeatures];
  b1 =  new float[nnStructure->numHiddenNeurons];

  mlp_w2  =  new float *[nnStructure->numClasses];
  for(i=0;i<nnStructure->numClasses;i++)
      mlp_w2[i] =  new float[nnStructure->numHiddenNeurons];
  b2 =  new float[nnStructure->numClasses];
  return 1;
}
unsigned int NN::MLP_InitTrain(void){
  unsigned int i, j;
  srand( (unsigned)time( NULL ) );
  for(i=0;i<nnStructure->numHiddenNeurons;i++)
  {
	for(j=0;j<nnStructure->numFeatures;j++)
       mlp_w1[i][j]=(float)(rand()-RAND_MAX*0.5)/(RAND_MAX*nnTraining->scale);
    b1[i]=(float)(rand()-RAND_MAX*0.5)/(RAND_MAX*nnTraining->scale);
  }
  for(i=0;i<nnStructure->numClasses;i++)
  {
    for(j=0;j<nnStructure->numHiddenNeurons;j++)
       mlp_w2[i][j]=(float)(rand()-RAND_MAX*0.5)/(RAND_MAX*nnTraining->scale);
    b2[i]=(float)(rand()-RAND_MAX*0.5)/(RAND_MAX*nnTraining->scale);
  }

  return 1;
}

 unsigned int NN::MLP_Train()
{
  unsigned int i,j,k,l,m;
  float E,alpha,ita;
  float **delta_mlp_w1=new float *[nnStructure->numHiddenNeurons];
  MLP_Init();
  MLP_InitTrain();
  for(i=0;i<nnStructure->numHiddenNeurons;i++)
      delta_mlp_w1[i]=new float[nnStructure->numFeatures];
  float *delta_b1=new float[nnStructure->numHiddenNeurons];
  for(i=0;i<nnStructure->numHiddenNeurons;i++)
	 {
	  for(j=0;j<nnStructure->numFeatures;j++)
	delta_mlp_w1[i][j]=0;
      delta_b1[i]=0;
  }
  float **delta_mlp_w2=new float *[nnStructure->numClasses];
  for(i=0;i<nnStructure->numClasses;i++)
      delta_mlp_w2[i]=new float[nnStructure->numHiddenNeurons];
  float *delta_b2=new float[nnStructure->numClasses];
  for(i=0;i<nnStructure->numClasses;i++)
	 {
	  for(j=0;j<nnStructure->numHiddenNeurons;j++)
	delta_mlp_w2[i][j]=0;
      delta_b2[i]=0;
  }


  float *delta1= new float[nnStructure->numHiddenNeurons];
  for(i=0;i<nnStructure->numHiddenNeurons;i++)delta1[i]=0;

  float *delta2= new float[nnStructure->numClasses];
  for(i=0;i<nnStructure->numClasses;i++)delta2[i]=0;
  float *I0    = new float[nnStructure->numFeatures];
  float *o1    = new float[nnStructure->numHiddenNeurons];
  float *o2    = new float[nnStructure->numClasses];  // output of the output layer;
  float **target    =    new float *[nnStructure->numClasses];
  
  for(i=0;i<nnStructure->numClasses;i++)
  {   
	  target[i]= new float[nnStructure->numClasses];
	  for(j=0;j<nnStructure->numClasses;j++)
	  	  target[i][j]=nnTraining->dLow;
      target[i][i]=nnTraining->dHigh;
  }
  unsigned int epoch;
  unsigned int max_l=100;
 
// start training
 for(epoch=0;epoch<nnTraining->maxEpoch;epoch++)
 {
  
   if(max_l==0)
     break;
   max_l=0;
   for(unsigned int classNum=0;classNum<nnStructure->numClasses;classNum++)
   {
   
      for(i=0;i<nnStructure->numFeatures;i++)
	  {  
	    I0[i]=0;
	    for(j=0;j<nnTraining->numObservations[classNum];j++)   
     	   I0[i]=I0[i]+nnTraining->trainData[classNum][j][i];
	    I0[i]/=(float)(nnTraining->numObservations[classNum]);
	  }  
      for(l=0;l<nnTraining->maxLoop;l++)
	  {
         alpha= (1+0.5/(l +1))*nnTraining->alpha;
	     ita=(1+0.5/(l+1))*nnTraining->ita;
	   	 for(i=0;i<nnStructure->numHiddenNeurons;i++)
		 {
	       o1[i]=b1[i];
	       for(m=0;m<nnStructure->numFeatures;m++)
	           o1[i]+=mlp_w1[i][m]*I0[m]; //I1=mlp_w1*I0+b1;
		 }
         for(i=0;i<nnStructure->numHiddenNeurons;i++)
	     o1[i]=logsig(o1[i]);//o1=logsig(I1);
         for(i=0;i<nnStructure->numClasses;i++)
		 {
	       o2[i]=b2[i];
	       for(m=0;m<nnStructure->numHiddenNeurons;m++)
	           o2[i]+=mlp_w2[i][m]*o1[m];
		 }
	    //I2=mlp_w2*o1+b2;
         for(i=0;i<nnStructure->numClasses;i++)
	      	 o2[i]=logsig(o2[i]);//o2=logsig(I2);
	     E=0;
         for(i=0;i<nnStructure->numClasses;i++)
         E=E+(o2[i]-target[i][classNum])*(o2[i]-target[i][classNum]);
         E=sqrt(E/nnStructure->numClasses);
         if(E<=nnTraining->error)
	        	break;
	     for(i=0;i<nnStructure->numClasses;i++)
         delta2[i]=2*(target[i][classNum]-o2[i]);//*o2[i]*(1-o2[i]);
         for(i=0;i<nnStructure->numClasses;i++)
		 {
	         for(j=0;j<nnStructure->numHiddenNeurons;j++)
	           delta_mlp_w2[i][j] = alpha*delta2[i]*o1[j]+ ita*delta_mlp_w2[i][j];
	         delta_b2[i] = alpha*delta2[i]*1.0  + ita*delta_b2[i];
		 }
         for(i=0;i<nnStructure->numHiddenNeurons;i++)
		 {
	         delta1[i]=0;
	         for(k=0;k<nnStructure->numClasses;k++)
	            delta1[i]+=delta2[k]*mlp_w2[k][i];
	         delta1[i]=delta1[i]*o1[i]*(1-o1[i]);
	         for(j=0;j<nnStructure->numFeatures;j++)
	             delta_mlp_w1[i][j]=alpha*delta1[i]*I0[j]+ita*delta_mlp_w1[i][j];
	         delta_b1[i]= alpha*delta1[i]*1.0+ita*delta_b1[i];
		 }

         for(i=0;i<nnStructure->numHiddenNeurons;i++)
		 {
 		    for(j=0;j<nnStructure->numFeatures;j++)
	           mlp_w1[i][j]+=delta_mlp_w1[i][j];//nnStructure->mlp_w1=nnStructure->mlp_w1+delta_mlp_w1;
            b1[i]+=delta_b1[i];//nnStructure->b1=nnStructure->b1+delta_b1;
		 }
         for(i=0;i<nnStructure->numClasses;i++)
		 {
 		    for(j=0;j<nnStructure->numHiddenNeurons;j++)
	            mlp_w2[i][j]+=delta_mlp_w2[i][j];//nnStructure->mlp_w2=nnStructure->mlp_w2+delta_mlp_w2;
            b2[i]+=delta_b2[i];//nnStructure->b2=nnStructure->b2+delta_b2;
		 }
	  }//loop l end
   if(l>max_l)
	  max_l=l;
   
   }  //loop classNum end 
 }  //loop epoch end 
 if(epoch==nnTraining->maxEpoch)
 {
	MLP_memfree();
	ErrorMessage("MLP Training Failed!");
 }  
 
 for(i=0;i<nnStructure->numHiddenNeurons;i++)
    delete[] delta_mlp_w1[i];
 delete[] delta_mlp_w1;
 delete[] delta_b1;
 for(i=0;i<nnStructure->numClasses;i++)
    delete[] delta_mlp_w2[i];
 delete[] delta_mlp_w2;
 delete[] delta_b2;
 delete[] delta1;
 delete[] delta2;
 delete[] I0    ;
 delete[] o1    ;
 delete[] o2    ;
 for(i=0;i<nnStructure->numClasses;i++)
	  delete[] target[i];
  delete[] target;
  
 return 1;
}

 unsigned int NN::MLP_Recognize(float **input,unsigned int inputLength)
{
    unsigned int i,j,result;
    float *I0=new float[nnStructure->numFeatures];
	for(i=0;i<nnStructure->numFeatures;i++)
	{
		I0[i]=0;
        	for(j=0;j<inputLength;j++)
		  I0[i]=I0[i]+input[j][i];
	   	I0[i]/=(float)inputLength;
	}
	float *o1=new float[nnStructure->numHiddenNeurons];
    float *o2=new float[nnStructure->numClasses];
        
    for(i=0;i<nnStructure->numHiddenNeurons;i++)
	{
	     o1[i]=b1[i];
	     for(j=0;j<nnStructure->numFeatures;j++)
	          o1[i]+=mlp_w1[i][j]*I0[j]; //I1=mlp_w1*I0+b1;
	}
    for(i=0;i<nnStructure->numHiddenNeurons;i++)
	   o1[i]=logsig(o1[i]);//o1=logsig(I1);
    for(i=0;i<nnStructure->numClasses;i++)
	{
	     o2[i]=b2[i];
	     for(j=0;j<nnStructure->numHiddenNeurons;j++)
	         o2[i]+=mlp_w2[i][j]*o1[j];
	}
	//I2=mlp_w2*o1+b2;
    for(i=0;i<nnStructure->numClasses;i++)
	   	 o2[i]=logsig(o2[i]);//o2=logsig(I2);
     //find the maximal output node
  
	float temp=0;
    for(i=0;i<nnStructure->numClasses;i++)
	{
	    if(temp<o2[i])
		{
	      temp=o2[i];
	      result  =   i;
		}
	}
   
  delete[] I0;
  delete[] o1;
  delete[] o2;
  return result;
}
unsigned int NN::MLP_Save(FILE *fp){
        unsigned int i, j, k;
        nnStructure->Dump(fp);
        fprintf(fp, "\n");
        assert(mlp_w1);
        for(j=0;j<nnStructure->numHiddenNeurons;j++){
          assert(mlp_w1[j]);
          for(k=0;k<nnStructure->numFeatures;k++){
                fprintf(fp, "%f\t",mlp_w1[j][k]);
          }
        }
        assert(mlp_w2);
        for(j=0;j<nnStructure->numClasses;j++){
          assert(mlp_w2[j]);
          for(k=0;k<nnStructure->numHiddenNeurons;k++){
           assert(mlp_w2[j][k]);
                fprintf(fp, "%f\t",mlp_w2[j][k]);
         }
        }
        assert(b1);
        for(i=0;i<nnStructure->numHiddenNeurons;i++) {
                fprintf(fp, "%f\t",b1[i]);
        }
        assert(b2);
        for(i=0;i<nnStructure->numClasses;i++) {
                fprintf(fp, "%f\t",b2[i]);
        }

        return 1;
}
unsigned int NN::MLP_Load(FILE *fp){
        unsigned int i, j, k;
	MLP_Init();
	nnStructure->Read(fp);
        assert(mlp_w1);
        for(j=0;j<nnStructure->numHiddenNeurons;j++){
          assert(mlp_w1[j]);
          for(k=0;k<nnStructure->numFeatures;k++){
                fscanf(fp, "%f\t",&mlp_w1[j][k]);
          }
        }
        assert(mlp_w2);
        for(j=0;j<nnStructure->numClasses;j++){
          assert(mlp_w2[j]);
          for(k=0;k<nnStructure->numHiddenNeurons;k++){
           assert(mlp_w2[j][k]);
                fscanf(fp, "%f\t",&mlp_w2[j][k]);
         }
        }
        assert(b1);
        for(i=0;i<nnStructure->numHiddenNeurons;i++) {
                fscanf(fp, "%f\t",&b1[i]);
        }
        assert(b2);
        for(i=0;i<nnStructure->numClasses;i++) {
                fscanf(fp, "%f\t",&b2[i]);
        }

        return 1;
}
void NN::MLP_memfree(void)
{
  unsigned int i;	
  for(i=0;i<nnStructure->numHiddenNeurons;i++)
      delete[] mlp_w1[i];
  delete[] mlp_w1;
  delete[] b1;
  for(i=0;i<nnStructure->numClasses;i++)
      delete[] mlp_w2[i];
  delete[] mlp_w2;
  delete[] b2;
  return;
}

 unsigned int NN::PER_Init(void)
{
  unsigned int i;
 
  mlp_w1  =  new float *[nnStructure->numClasses];
  for(i=0;i<nnStructure->numClasses;i++)
      mlp_w1[i] =  new float[nnStructure->numFeatures];
  b1 =  new float[nnStructure->numClasses];
  return 1;
}
unsigned int NN::PER_InitTrain(void) {
  unsigned int i, j;
  srand( (unsigned)time( NULL ) );
  for(i=0;i<nnStructure->numClasses;i++)
  {
	for(j=0;j<nnStructure->numFeatures;j++)
       mlp_w1[i][j]=(float)(rand()-RAND_MAX*0.5)/(RAND_MAX*nnTraining->scale);
    b1[i]=(float)(rand()-RAND_MAX*0.5)/(RAND_MAX*nnTraining->scale);
  }
  return 1;
}


 unsigned int NN::PER_Train()
{
  unsigned int i,j,l,m;
  float E,alpha,ita;
  PER_Init();
  PER_InitTrain();
  float **delta_mlp_w1=new float *[nnStructure->numClasses];
  for(i=0;i<nnStructure->numClasses;i++)
      delta_mlp_w1[i]=new float[nnStructure->numFeatures];
  float *delta_b1=new float[nnStructure->numClasses];
  for(i=0;i<nnStructure->numClasses;i++)
	 {
	  for(j=0;j<nnStructure->numFeatures;j++)
	delta_mlp_w1[i][j]=0;
      delta_b1[i]=0;
  }


  float *delta1= new float[nnStructure->numClasses];
  for(i=0;i<nnStructure->numClasses;i++)delta1[i]=0;
  float *I0    = new float[nnStructure->numFeatures];
  float *o1    = new float[nnStructure->numClasses];  // output of the output layer;
  float **target    =    new float *[nnStructure->numClasses];
  
  for(i=0;i<nnStructure->numClasses;i++)
  {   
	  target[i]= new float[nnStructure->numClasses];
	  for(j=0;j<nnStructure->numClasses;j++)
	  	  target[i][j]=nnTraining->dLow;
      target[i][i]=nnTraining->dHigh;
  }

  unsigned int epoch;
  unsigned int max_l=100;
 
// start training
 for(epoch=0;epoch<nnTraining->maxEpoch;epoch++)
 {
  
   if(max_l==0)
     break;
   max_l=0;
   for(unsigned int classNum=0;classNum<nnStructure->numClasses;classNum++)
   {
   
      for(i=0;i<nnStructure->numFeatures;i++)
	  {  
	    I0[i]=0;
	    for(j=0;j<nnTraining->numObservations[classNum];j++)   
     	   I0[i]=I0[i]+nnTraining->trainData[classNum][j][i];
	    I0[i]/=(float)(nnTraining->numObservations[classNum]);
	  }  
      for(l=0;l<nnTraining->maxLoop;l++)
	  {
         alpha= (1+0.5/(l +1))*nnTraining->alpha;
	     ita=(1+0.5/(l+1))*nnTraining->ita;
	   	 for(i=0;i<nnStructure->numClasses;i++)
		 {
	       o1[i]=b1[i];
	       for(m=0;m<nnStructure->numFeatures;m++)
	           o1[i]+=mlp_w1[i][m]*I0[m]; //I1=mlp_w1*I0+b1;
		 }
         for(i=0;i<nnStructure->numClasses;i++)
	         o1[i]=logsig(o1[i]);//o1=logsig(I1);
         E=0;
         for(i=0;i<nnStructure->numClasses;i++)
             E=E+(o1[i]-target[i][classNum])*(o1[i]-target[i][classNum]);
         E=sqrt(E/nnStructure->numClasses);
         if(E<=nnTraining->error)
	        	break;
	     for(i=0;i<nnStructure->numClasses;i++)
         delta1[i]=2*(target[i][classNum]-o1[i]);//*o2[i]*(1-o2[i]);
         for(i=0;i<nnStructure->numClasses;i++)
		 {
	         for(j=0;j<nnStructure->numFeatures;j++)
	           delta_mlp_w1[i][j] = alpha*delta1[i]*I0[j]+ ita*delta_mlp_w1[i][j];
	         delta_b1[i] = alpha*delta1[i]*1.0  + ita*delta_b1[i];
		 }
         
         for(i=0;i<nnStructure->numClasses;i++)
		 {
 		    for(j=0;j<nnStructure->numFeatures;j++)
	           mlp_w1[i][j]+=delta_mlp_w1[i][j];//nnStructure->mlp_w1=nnStructure->mlp_w1+delta_mlp_w1;
            b1[i]+=delta_b1[i];//nnStructure->b1=nnStructure->b1+delta_b1;
		 }
	  }//loop l end
   if(l>max_l)
	  max_l=l;
   
   }  //loop classNum end 
 }  //loop epoch end 
 if(epoch==nnTraining->maxEpoch)
 {
	ErrorMessage("Perceptrons Training Failed!");
	PER_memfree();
 }  
 
 for(i=0;i<nnStructure->numClasses;i++)
    delete[] delta_mlp_w1[i];
 delete[] delta_mlp_w1;
 delete[] delta_b1;
 delete[] delta1;
 delete[] I0    ;
 delete[] o1    ;
 for(i=0;i<nnStructure->numClasses;i++)
	  delete[] target[i];
  delete[] target;
  
 return 1;

}

 unsigned int NN::PER_Recognize(float **input, unsigned int inputLength)
{
    unsigned int i,j,result;
    float *I0=new float[nnStructure->numFeatures];
	for(i=0;i<nnStructure->numFeatures;i++)
	{
		I0[i]=0;
        for(j=0;j<inputLength;j++)
		  I0[i]=I0[i]+input[j][i];
	   I0[i]/=(float)inputLength;
	}
	float *o1=new float[nnStructure->numClasses];
    
    for(i=0;i<nnStructure->numClasses;i++)
	{
	     o1[i]=b1[i];
	     for(j=0;j<nnStructure->numFeatures;j++)
	          o1[i]+=mlp_w1[i][j]*I0[j]; //I1=mlp_w1*I0+b1;
	}
    for(i=0;i<nnStructure->numClasses;i++)
	   o1[i]=logsig(o1[i]);//o1=logsig(I1);
     //find the maximal output node
  
	float temp=0;
    for(i=0;i<nnStructure->numClasses;i++)
	{
	    if(temp<o1[i])
		{
	      temp=o1[i];
	      result  =   i;
		}
	}
   
  delete[] I0;
  delete[] o1;
  return result;

}
unsigned int NN::PER_Save(FILE *fp){
        unsigned int i, j, k;
        nnStructure->Dump(fp);
        fprintf(fp, "\n");
        assert(mlp_w1);
        for(j=0;j<nnStructure->numClasses;j++){
          assert(mlp_w1[j]);
          for(k=0;k<nnStructure->numFeatures;k++){
                fprintf(fp, "%f\t",mlp_w1[j][k]);
          }
        }

        assert(b1);
        for(i=0;i<nnStructure->numClasses;i++) {
                fprintf(fp, "%f\t",b1[i]);
        }
        return 1;
}
unsigned int NN::PER_Load(FILE *fp){
        unsigned int i, j, k;
	PER_Init();
	nnStructure->Read(fp);
        assert(mlp_w1);
        for(j=0;j<nnStructure->numClasses;j++){
          assert(mlp_w1[j]);
          for(k=0;k<nnStructure->numFeatures;k++){
                fscanf(fp, "%f\t",&mlp_w1[j][k]);
          }
        }
        assert(b1);
        for(i=0;i<nnStructure->numClasses;i++) {
                fscanf(fp, "%f\t",&b1[i]);
        }
        return 1;
}
void NN::PER_memfree(void)
{
  unsigned int i;	
  for(i=0;i<nnStructure->numClasses;i++)
      delete[] mlp_w1[i];
  delete[] mlp_w1;
  delete[] b1;
  return;
}
