/* This file is part of the implementation of the Dynamic Neuaral Network
   for speech recognition.

   Copyright (C) 1998 Lin Zhong<lzhong@princeton.edu>
                      Runsheng Liu <lrs-dee@tsinghua.edu.cn>
                      Circuits & Systems Group
                      Dept. of Electronic Engineering
                      Tsinghua University, Beijing
   Copyright (C) 2004 Lin Zhong <lzhong@princeton.edu>
                      Computer Engineering Group
                      Dept. of Electrical Engineering
                      Princeton University

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software. In case the
software is used in a publication, please cite
Lin Zhong, Yuanyuan Shi and Runsheng Liu,
"A Dynamic Neural Network for Syllable Recognition",
in Proc. Intl. Joint Conf. Neural Networks, Washington D.C., 1999.


THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/
#ifndef NN_H
#define NN_H	
#include <stdlib.h>
#include <stdio.h>

#define MAX_CLASSES 60
#define ErrorMessage(x)	fprintf(stderr,"File %s line %d: %s",__FILE__,__LINE__, x)
typedef enum NN_Type{DNN=0, MLP=1, PER=2, SVD=3,Poly=4} NN_Type; 

typedef class nnStructParam{
public:

  unsigned int numHiddenNeurons;
  unsigned int numClasses;
/* For DNN only */
  unsigned int numFeatures; //# of features per input
  unsigned int inputTimeWindow;
  unsigned int hiddenLayerTimeWindow;
  unsigned int inputFeatureWindow;

  NN_Type type;

  nnStructParam(void);

  void Dump(FILE *fp);
  void Read(FILE *fp);
} nnStructParam;

typedef class nnTrainParam{
public:
  
  float error;
  unsigned int   maxEpoch;
  float alpha;
  float ita;
  unsigned int maxLoop;  
  float dLow;
  float dHigh;
  float scale;

  unsigned int numClasses;
  /* Currently one training sample per class */
  unsigned int numObservations[MAX_CLASSES];
  /* three-dimension array*/
  /* first: pattern class num */
  /* second: frame num of the sample*/
  /* third: feature num for a frame */
  float **trainData[MAX_CLASSES];

  void ShowParameters(void);
  nnTrainParam(void);
  ~nnTrainParam();
} nnTrainParam;

typedef class NN  
{
private:
	 nnStructParam *nnStructure ;
	 nnTrainParam *nnTraining;

	 float ** mlp_w2;
	 float ** mlp_w1;

	 float * b2;
	 float * b1;
	 float **** w2;
	 float *** w1;

	 void PER_memfree(void);
	 unsigned int PER_InitTrain(void);
	 unsigned int PER_Init(void);
	 unsigned int PER_Train(void);
	 unsigned int PER_Recognize(float **input,unsigned int inputLength);
	 unsigned int PER_Save(FILE *fp);
	 unsigned int PER_Load(FILE *fp);

	 void MLP_memfree(void);
	 unsigned int MLP_InitTrain(void);
	 unsigned int MLP_Init(void);
	 unsigned int MLP_Train(void);
	 unsigned int MLP_Recognize(float **input, unsigned int inputLength);
	 unsigned int MLP_Save(FILE *fp);
	 unsigned int MLP_Load(FILE *fp);

	 void DNN_memfree(void);
	 float logsig(float x);
	 unsigned int DNN_InitTrain(void);
	 unsigned int DNN_Init(void);
	 unsigned int DNN_Train(void);
	 unsigned int DNN_Recognize(float **input, unsigned int inputLength);
	 unsigned int DNN_Save(FILE *fp);
	 unsigned int DNN_Load(FILE *fp);
	
public:
	NN(nnStructParam &StructParam,nnTrainParam &TrainParam );
	NN(nnStructParam &StructParam);
	~NN();

	unsigned int NNRecognize(float **input, unsigned int inputLength);
	unsigned int NNTrain(void);
	unsigned int NNSave(char *fileName);
	void NNLoad(char *fileName);
	void SetMethods(NN_Type type);

        void (NN::*memfree)(void);
        unsigned int (NN::*Train)(void);
        unsigned int (NN::*Recognize)(float **input, unsigned int inputLength);
	unsigned int (NN::*Save)(FILE *fp);
	unsigned int (NN::*Load)(FILE *fp);

} NN;
#endif
