/* This file is part of the implementation of the Dynamic Neuaral Network
   for speech recognition.

   Copyright (C) 1998 Lin Zhong<lzhong@princeton.edu>
                      Runsheng Liu <lrs-dee@tsinghua.edu.cn>
                      Circuits & Systems Group
                      Dept. of Electronic Engineering
                      Tsinghua University, Beijing
   Copyright (C) 2004 Lin Zhong <lzhong@princeton.edu>
                      Computer Engineering Group
                      Dept. of Electrical Engineering
                      Princeton University

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software. In case the
software is used in a publication, please cite
Lin Zhong, Yuanyuan Shi and Runsheng Liu,
"A Dynamic Neural Network for Syllable Recognition",
in Proc. Intl. Joint Conf. Neural Networks, Washington D.C., 1999.


THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <unistd.h>
#include "TrainNN.h"
#include "SpeechProc.h"
#include "NN.h"
static char *NNTypeStr = "DNN";
static unsigned int inputTimeWindow= 5;
static unsigned int inputFeatureWindow= 8;
static unsigned int hiddenTimeWindow= 5;
static unsigned int hidden= 12;
static char nnDataFileStr[1024] = "./nn.dat";
static char dataInfoStr[1024] = "./speech.info";

static const char *arguments = "T:i:f:h:H:N:S:";
static char usage[] = { "train\n\t-T name: type of NN: DNN(default), MLP, or PER\n\t-i num: size of input time window for DNN(default 5)\n\t-f num: size of input feature window for DNN(default:8)\n\t-h num: size of hidden layer time window for DNN(default 5)\n\t-H num: num of hidden neurons(MLP) or hidden layers(DNN)(default 12)\n\t-N filename: name for the file to store NN data(default:./nn.dat)\n\t-S filename: file for info. about training data(default:./speech.info)"};

TrainNN::TrainNN(void) {
	return;
}
TrainNN::~TrainNN(){
	return;	
}
int TrainNN::GetConfiguration(int argc, char *argv[]){
	char i;
	extern char *optarg;
	while((i = getopt(argc, argv, arguments))!=EOF){
		switch(i) {
		  case 'T':
			sprintf(NNTypeStr,"%s",optarg);
			break;
		  case 'i':
			inputTimeWindow = atoi(optarg);
			break;
		  case 'f':
			inputFeatureWindow = atoi(optarg);
			break;
		  case 'h':
			hiddenTimeWindow = atoi(optarg);
			break;
		  case 'H':
			hidden = atoi(optarg);
			break;
		  case 'N':
			sprintf(nnDataFileStr, "%s",optarg);
			break;
		  case 'S':
			sprintf(dataInfoStr, "%s", optarg);
			break;
		  default:
			fprintf(stderr,"%s\n",usage);
			exit(-1);
		}						
	}

	if(strcmp(NNTypeStr,"MLP")==0) {
        	StructParam.type = MLP;
	} else if(strcmp(NNTypeStr,"PER")==0){
		StructParam.type = PER;
	} else {
		StructParam.type = DNN;
	}
        StructParam.inputTimeWindow = inputTimeWindow;
        StructParam.inputFeatureWindow = inputFeatureWindow;
        StructParam.hiddenLayerTimeWindow = hiddenTimeWindow;
        StructParam.numHiddenNeurons = hidden;	
        StructParam.numFeatures = FeatureParam.Cepstrum_order;

	sprintf(NNDataFile, "%s", nnDataFileStr);
	sprintf(dataInfo,"%s",dataInfoStr);
	return 1;
}

int TrainNN::ReadData(void) {
	SpeechProc getFeatures(FeatureParam);
	FILE *fp;
	char fileName[1024];
	int classNum = 0;
	int numClasses = 0;
	short *voice;
	unsigned int voiceLength;
	if((fp=fopen(dataInfo,"r"))==NULL){
		ErrorMessage("Cannot open training data info file");
	}
	while(feof(fp)==0){
		fscanf(fp,"%d", &classNum);
		if(classNum>numClasses)numClasses = classNum;
		fscanf(fp,"%s\n", fileName);
		printf("Read file %s for class %d\n", fileName, classNum);
		voice = getFeatures.ReadWav(fileName, voiceLength);
		TrainParam.trainData[classNum] = getFeatures.FeatureExtract(voice, voiceLength);
		//sprintf(fileName, "%s.txt", fileName);
		//getFeatures.SaveFeatures(fileName);
		TrainParam.numObservations[classNum] = getFeatures.frameNum;
	}
	numClasses++;
	TrainParam.numClasses = numClasses;
	StructParam.numClasses = numClasses;
	fclose(fp);
	fprintf(stderr, "Training data read\n");
	return 1;
}
int TrainNN::Train(void){
	NN NNet(StructParam, TrainParam);
	NNet.NNTrain();
	fprintf(stderr,"NN trained\n");
	NNet.NNSave(NNDataFile);
	fprintf(stderr, "NN saved\n");
	return 1;
}
