/* This file is part of the implementation of the Dynamic Neuaral Network
   for speech recognition.

   Copyright (C) 1998 Lin Zhong<lzhong@princeton.edu>
                      Runsheng Liu <lrs-dee@tsinghua.edu.cn>
                      Circuits & Systems Group
                      Dept. of Electronic Engineering
                      Tsinghua University, Beijing
   Copyright (C) 2004 Lin Zhong <lzhong@princeton.edu>
                      Computer Engineering Group
                      Dept. of Electrical Engineering
                      Princeton University

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software. In case the
software is used in a publication, please cite
Lin Zhong, Yuanyuan Shi and Runsheng Liu,
"A Dynamic Neural Network for Syllable Recognition",
in Proc. Intl. Joint Conf. Neural Networks, Washington D.C., 1999.


THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "NN.h"
#include "SpeechProc.h"

const static char *arguments= "N:S:";
static char nnDataFileStr[1024] = "./nn.dat";
static char dataInfoStr[1024] = "./testspeech.info";

static char usage[] = {"recog\n\t-N filename: file to load NN data\n\t\t(default:./nn.dat)\n\t-S filename: file for info. about testing data\n\t\t(default: ./testspeech.info)"};

int main(int argc, char *argv[]) {
	unsigned int id;
	unsigned int result;
	char wavFile[1024];
	float **features;
	featureParam FeatureParam;
	nnStructParam StructParam;
	NN NNet(StructParam);
	FILE *fp;
	short *voiceData;
	unsigned int voiceLength;
	extern char *optarg;
	char i;
	while((i = getopt(argc, argv, arguments))!=EOF){
		switch(i) {
                  case 'N':
                        sprintf(nnDataFileStr, "%s",optarg);
                        break;

                  case 'S':
                        sprintf(dataInfoStr, "%s", optarg);
                        break;
                  default:
                        fprintf(stderr,"%s\n",usage);
			return -1;
                }
        }

	SpeechProc getFeatures(FeatureParam);


	NNet.NNLoad(nnDataFileStr);
	
	if((fp=fopen(dataInfoStr,"r"))==NULL) {
		fprintf(stderr,"%s: ", dataInfoStr);
		ErrorMessage("Cannot open test data info file");
	}
	while(feof(fp)==0){
		fscanf(fp,"%u", &id);
		fscanf(fp,"%s\n", wavFile);
		voiceData = getFeatures.ReadWav(wavFile,voiceLength );
		features = getFeatures.FeatureExtract(voiceData, voiceLength);
		result = NNet.NNRecognize(features, getFeatures.GetFrameNum());
		getFeatures.ReleaseFeatures();
		fprintf(stderr, "File %s (%d) is recognized as %d\n",wavFile,id, result);
	}

	return 1;
}
